// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "GetDataRefTask.h"

#include "xplcommon/Util.h"

//------------------------------------------------------------------------------

using xplra::GetByteArrayDataRefTask;

using xplcommon::Util;

using std::string;

//------------------------------------------------------------------------------

GetByteArrayDataRefTask::GetByteArrayDataRefTask(const std::string& name,
                                                 int maxBytes, int offset) :
    DataRefTask(name),
    maxBytes(maxBytes),
    offset(offset),
    data((maxBytes>0) ? new unsigned char[maxBytes] : 0),
    length(-1)
{
}

//------------------------------------------------------------------------------

GetByteArrayDataRefTask::GetByteArrayDataRefTask(XPLMDataRef dataRef,
                                                 int maxBytes, int offset) :
    DataRefTask(dataRef),
    maxBytes(maxBytes),
    offset(offset),
    data((maxBytes>0) ? new unsigned char[maxBytes] : 0),
    length(-1)
{
}

//------------------------------------------------------------------------------

GetByteArrayDataRefTask::~GetByteArrayDataRefTask()
{
    delete [] data;
}

//------------------------------------------------------------------------------

void GetByteArrayDataRefTask::process()
{
    if (maxBytes<0) {
        maxBytes = XPLMGetDatab(getDataRef(), 0, 0, 0);

        if (maxBytes>0) {
            maxBytes -= offset;
        }

        if (maxBytes>0) {
            data = new unsigned char[maxBytes];
        } else {
            maxBytes = 0;
        }
    }

    Util::debug("GetByteArrayDataRefTask::process: dataRef=%p, maxBytes=%d, data=%p, offset\n",
                getDataRef(), maxBytes, data, offset);
    if (maxBytes>0) {
        length = XPLMGetDatab(getDataRef(), data, offset, maxBytes);
    } else {
        length = -1;
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
